@@ -12,7 +12,10 @@
 # License for the specific language governing permissions and limitations under
 # the License.
 
+from stevedore import driver
+
 from oslo_log import log as logging
+from oslo_utils import netutils
 
 from zaqar.common.api import errors as api_errors
 from zaqar.common.api import response
@@ -31,6 +34,7 @@ class Endpoints(object):
         self._queue_controller = storage.queue_controller
         self._message_controller = storage.message_controller
         self._claim_controller = storage.claim_controller
+        self._subscription_controller = storage.subscription_controller
 
         self._pools_controller = control.pools_controller
         self._flavors_controller = control.flavors_controller
@@ -38,9 +42,10 @@ class Endpoints(object):
         self._validate = validate
 
         self._defaults = defaults
+        self._subscription_url = None
 
     # Queues
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def queue_list(self, req):
         """Gets a list of queues
 
@@ -60,6 +65,8 @@ class Endpoints(object):
             self._validate.queue_listing(**kwargs)
             results = self._queue_controller.list(
                 project=project_id, **kwargs)
+            # Buffer list of queues. Can raise NoPoolFound error.
+            queues = list(next(results))
         except (ValueError, validation.ValidationFailed) as ex:
             LOG.debug(ex)
             headers = {'status': 400}
@@ -68,10 +75,7 @@ class Endpoints(object):
             LOG.exception(ex)
             error = 'Queues could not be listed.'
             headers = {'status': 503}
-            return api_utils.error_response(req, ex, error, headers)
-
-        # Buffer list of queues
-        queues = list(next(results))
+            return api_utils.error_response(req, ex, headers, error)
 
         # Got some. Prepare the response.
         body = {'queues': queues}
@@ -79,7 +83,7 @@ class Endpoints(object):
 
         return response.Response(req, body, headers)
 
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def queue_create(self, req):
         """Creates a queue
 
@@ -99,6 +103,7 @@ class Endpoints(object):
         try:
             self._validate.queue_identification(queue_name, project_id)
             self._validate.queue_metadata_length(len(str(metadata)))
+            self._validate.queue_metadata_putting(metadata)
             created = self._queue_controller.create(queue_name,
                                                     metadata=metadata,
                                                     project=project_id)
@@ -116,7 +121,7 @@ class Endpoints(object):
             headers = {'status': 201} if created else {'status': 204}
             return response.Response(req, body, headers)
 
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def queue_delete(self, req):
         """Deletes a queue
 
@@ -142,7 +147,7 @@ class Endpoints(object):
             headers = {'status': 204}
             return response.Response(req, body, headers)
 
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def queue_get(self, req):
         """Gets a queue
 
@@ -176,7 +181,7 @@ class Endpoints(object):
             headers = {'status': 200}
             return response.Response(req, body, headers)
 
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def queue_get_stats(self, req):
         """Gets queue stats
 
@@ -217,8 +222,59 @@ class Endpoints(object):
             headers = {'status': 200}
             return response.Response(req, body, headers)
 
+    @api_utils.on_exception_sends_500
+    def queue_purge(self, req):
+        """Purge queue
+
+        :param req: Request instance ready to be sent.
+        :type req: `api.common.Request`
+        :return: resp: Response instance
+        :type: resp: `api.common.Response`
+        """
+        project_id = req._headers.get('X-Project-ID')
+        queue_name = req._body.get('queue_name')
+        resource_types = req._body.get('resource_types', ["messages",
+                                                          "subscriptions"])
+
+        LOG.debug(u'Purge queue - queue: %(queue)s, '
+                  u'project: %(project)s',
+                  {'queue': queue_name, 'project': project_id})
+
+        try:
+            pop_limit = 100
+            if "messages" in resource_types:
+                LOG.debug("Purge all messages under queue %s" % queue_name)
+                resp = self._pop_messages(req, queue_name,
+                                          project_id, pop_limit)
+                while resp.get_response()['body']['messages']:
+                    resp = self._pop_messages(req, queue_name,
+                                              project_id, pop_limit)
+
+            if "subscriptions" in resource_types:
+                LOG.debug("Purge all subscriptions under queue %s" %
+                          queue_name)
+                resp = self._subscription_controller.list(queue_name,
+                                                          project=project_id)
+                subscriptions = list(next(resp))
+                for sub in subscriptions:
+                    self._subscription_controller.delete(queue_name,
+                                                         sub['id'],
+                                                         project=project_id)
+
+        except storage_errors.QueueDoesNotExist as ex:
+            LOG.exception(ex)
+            headers = {'status': 404}
+            return api_utils.error_response(req, ex, headers)
+        except storage_errors.ExceptionBase as ex:
+            LOG.exception(ex)
+            headers = {'status': 503}
+            return api_utils.error_response(req, ex, headers)
+        else:
+            headers = {'status': 204}
+            return response.Response(req, {}, headers)
+
     # Messages
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def message_list(self, req):
         """Gets a list of messages on a queue
 
@@ -270,7 +326,7 @@ class Endpoints(object):
 
         return response.Response(req, body, headers)
 
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def message_get(self, req):
         """Gets a message from a queue
 
@@ -307,7 +363,7 @@ class Endpoints(object):
 
         return response.Response(req, body, headers)
 
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def message_get_many(self, req):
         """Gets a set of messages from a queue
 
@@ -345,7 +401,7 @@ class Endpoints(object):
 
         return response.Response(req, body, headers)
 
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def message_post(self, req):
         """Post a set of messages to a queue
 
@@ -370,18 +426,41 @@ class Endpoints(object):
             return api_utils.error_response(req, ex, headers, error)
 
         try:
+            # NOTE(flwang): Replace 'exists' with 'get_metadata' won't impact
+            # the performance since both of them will call
+            # collection.find_one()
+            queue_meta = None
+            try:
+                queue_meta = self._queue_controller.get_metadata(queue_name,
+                                                                 project_id)
+            except storage_errors.DoesNotExist as ex:
+                self._validate.queue_identification(queue_name, project_id)
+                self._queue_controller.create(queue_name, project=project_id)
+                # NOTE(flwang): Queue is created in lazy mode, so no metadata
+                # set.
+                queue_meta = {}
+
+            queue_max_msg_size = queue_meta.get('_max_messages_post_size',
+                                                None)
+            queue_default_ttl = queue_meta.get('_default_message_ttl', None)
+
+            # TODO(flwang): To avoid any unexpected regression issue, we just
+            # leave the _message_post_spec attribute of class as it's. It
+            # should be removed in Newton release.
+            if queue_default_ttl:
+                _message_post_spec = (('ttl', int, queue_default_ttl),
+                                      ('body', '*', None),)
+            else:
+                _message_post_spec = (('ttl', int, self._defaults.message_ttl),
+                                      ('body', '*', None),)
             # Place JSON size restriction before parsing
-            self._validate.message_length(len(str(messages)))
+            self._validate.message_length(len(str(messages)),
+                                          max_msg_post_size=queue_max_msg_size)
         except validation.ValidationFailed as ex:
             LOG.debug(ex)
             headers = {'status': 400}
             return api_utils.error_response(req, ex, headers)
 
-        _message_post_spec = (
-            ('ttl', int, self._defaults.message_ttl),
-            ('body', '*', None),
-        )
-
         try:
             messages = api_utils.sanitize(messages,
                                           _message_post_spec,
@@ -396,10 +475,6 @@ class Endpoints(object):
 
             self._validate.message_posting(messages)
 
-            if not self._queue_controller.exists(queue_name, project_id):
-                self._validate.queue_identification(queue_name, project_id)
-                self._queue_controller.create(queue_name, project=project_id)
-
             message_ids = self._message_controller.post(
                 queue_name,
                 messages=messages,
@@ -425,7 +500,7 @@ class Endpoints(object):
 
         return response.Response(req, body, headers)
 
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def message_delete(self, req):
         """Delete a message from a queue
 
@@ -476,7 +551,7 @@ class Endpoints(object):
 
         return response.Response(req, body, headers)
 
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def message_delete_many(self, req):
         """Deletes a set of messages from a queue
 
@@ -488,7 +563,7 @@ class Endpoints(object):
         project_id = req._headers.get('X-Project-ID')
         queue_name = req._body.get('queue_name')
         message_ids = req._body.get('message_ids')
-        pop_limit = req._body.get('pop_limit')
+        pop_limit = req._body.get('pop')
 
         LOG.debug(u'Messages collection DELETE - queue: %(queue)s,'
                   u'project: %(project)s, messages: %(message_ids)s',
@@ -509,7 +584,7 @@ class Endpoints(object):
         elif pop_limit:
             return self._pop_messages(req, queue_name, project_id, pop_limit)
 
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def _delete_messages_by_id(self, req, queue_name, ids, project_id):
         self._message_controller.bulk_delete(queue_name, message_ids=ids,
                                              project=project_id)
@@ -519,7 +594,7 @@ class Endpoints(object):
 
         return response.Response(req, body, headers)
 
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def _pop_messages(self, req, queue_name, project_id, pop_limit):
 
         LOG.debug(u'Pop messages - queue: %(queue)s, project: %(project)s',
@@ -540,7 +615,7 @@ class Endpoints(object):
         return response.Response(req, body, headers)
 
     # Claims
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def claim_create(self, req):
         """Creates a claim
 
@@ -606,7 +681,7 @@ class Endpoints(object):
 
         return response.Response(req, body, headers)
 
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def claim_get(self, req):
         """Gets a claim
 
@@ -651,7 +726,7 @@ class Endpoints(object):
 
         return response.Response(req, body, headers)
 
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def claim_update(self, req):
         """Updates a claim
 
@@ -698,7 +773,7 @@ class Endpoints(object):
             headers = {'status': 404}
             return api_utils.error_response(req, ex, headers, error)
 
-    @api_utils.raises_conn_error
+    @api_utils.on_exception_sends_500
     def claim_delete(self, req):
         """Deletes a claim
 
@@ -725,3 +800,170 @@ class Endpoints(object):
         body = _('Claim %s deleted.') % claim_id
 
         return response.Response(req, body, headers)
+
+    # Subscriptions
+    @api_utils.on_exception_sends_500
+    def subscription_list(self, req):
+        """List all subscriptions for a queue.
+
+        :param req: Request instance ready to be sent.
+        :type req: `api.common.Request`
+        :return: resp: Response instance
+        :type: resp: `api.common.Response`
+        """
+        project_id = req._headers.get('X-Project-ID')
+        queue_name = req._body.get('queue_name')
+
+        LOG.debug(u'Subscription list - project: %(project)s',
+                  {'project': project_id})
+
+        try:
+            kwargs = api_utils.get_headers(req)
+
+            self._validate.subscription_listing(**kwargs)
+            results = self._subscription_controller.list(
+                queue_name, project=project_id, **kwargs)
+            # Buffer list of subscriptions. Can raise NoPoolFound error.
+            subscriptions = list(next(results))
+        except (ValueError, validation.ValidationFailed) as ex:
+            LOG.debug(ex)
+            headers = {'status': 400}
+            return api_utils.error_response(req, ex, headers)
+        except storage_errors.ExceptionBase as ex:
+            LOG.exception(ex)
+            error = 'Subscriptions could not be listed.'
+            headers = {'status': 503}
+            return api_utils.error_response(req, ex, headers, error)
+
+        # Got some. Prepare the response.
+        body = {'subscriptions': subscriptions}
+        headers = {'status': 200}
+
+        return response.Response(req, body, headers)
+
+    @api_utils.on_exception_sends_500
+    def subscription_create(self, req, subscriber):
+        """Create a subscription for a queue.
+
+        :param req: Request instance ready to be sent.
+        :type req: `api.common.Request`
+        :return: resp: Response instance
+        :type: resp: `api.common.Response`
+        """
+        project_id = req._headers.get('X-Project-ID')
+        queue_name = req._body.get('queue_name')
+        options = req._body.get('options', {})
+        ttl = req._body.get('ttl', self._defaults.subscription_ttl)
+
+        LOG.debug(
+            u'Subscription create - queue: %(queue)s, project: %(project)s',
+            {'queue': queue_name,
+             'project': project_id})
+
+        try:
+            url = netutils.urlsplit(subscriber)
+            mgr = driver.DriverManager('zaqar.notification.tasks', url.scheme,
+                                       invoke_on_load=True)
+            req_data = req._env.copy()
+            mgr.driver.register(subscriber, options, ttl, project_id, req_data)
+
+            data = {'subscriber': subscriber,
+                    'options': options,
+                    'ttl': ttl}
+            self._validate.subscription_posting(data)
+            self._validate.queue_identification(queue_name, project_id)
+            if not self._queue_controller.exists(queue_name, project_id):
+                self._queue_controller.create(queue_name, project=project_id)
+            created = self._subscription_controller.create(queue_name,
+                                                           subscriber,
+                                                           data['ttl'],
+                                                           data['options'],
+                                                           project=project_id)
+        except validation.ValidationFailed as ex:
+            LOG.debug(ex)
+            headers = {'status': 400}
+            return api_utils.error_response(req, ex, headers)
+        except storage_errors.ExceptionBase as ex:
+            LOG.exception(ex)
+            error = _('Subscription %s could not be created.') % queue_name
+            headers = {'status': 503}
+            return api_utils.error_response(req, ex, headers, error)
+        else:
+            if created:
+                msg = _('Subscription %s created.') % queue_name
+                body = {'subscription_id': str(created), 'message': msg}
+                headers = {'status': 201}
+            else:
+                body = _('Subscription %s not created.') % queue_name
+                headers = {'status': 409}
+            return response.Response(req, body, headers)
+
+    @api_utils.on_exception_sends_500
+    def subscription_delete(self, req):
+        """Delete a specific subscription by ID.
+
+        :param req: Request instance ready to be sent.
+        :type req: `api.common.Request`
+        :return: resp: Response instance
+        :type: resp: `api.common.Response`
+        """
+        project_id = req._headers.get('X-Project-ID')
+        queue_name = req._body.get('queue_name')
+        subscription_id = req._body.get('subscription_id')
+
+        LOG.debug(
+            u'Subscription delete - queue: %(queue)s, project: %(project)s',
+            {'queue': queue_name, 'project': project_id})
+        try:
+            self._subscription_controller.delete(queue_name,
+                                                 subscription_id,
+                                                 project=project_id)
+        except storage_errors.ExceptionBase as ex:
+            LOG.exception(ex)
+            error = _('Subscription %(subscription)s for queue %(queue)s '
+                      'could not be deleted.') % {
+                'subscription': subscription_id, 'queue': queue_name}
+            headers = {'status': 503}
+            return api_utils.error_response(req, ex, headers, error)
+        else:
+            body = _('Subscription %s removed.') % subscription_id
+            headers = {'status': 204}
+            return response.Response(req, body, headers)
+
+    @api_utils.on_exception_sends_500
+    def subscription_get(self, req):
+        """Retrieve details about an existing subscription.
+
+        :param req: Request instance ready to be sent.
+        :type req: `api.common.Request`
+        :return: resp: Response instance
+        :type: resp: `api.common.Response`
+        """
+        project_id = req._headers.get('X-Project-ID')
+        queue_name = req._body.get('queue_name')
+        subscription_id = req._body.get('subscription_id')
+
+        LOG.debug(u'Subscription get - queue: %(queue)s, '
+                  u'project: %(project)s',
+                  {'queue': queue_name, 'project': project_id})
+
+        try:
+            resp_dict = self._subscription_controller.get(queue_name,
+                                                          subscription_id,
+                                                          project=project_id)
+        except storage_errors.DoesNotExist as ex:
+            LOG.debug(ex)
+            error = _('Subscription %(subscription)s for queue %(queue)s '
+                      'does not exist.') % {
+                'subscription': subscription_id, 'queue': queue_name}
+            headers = {'status': 404}
+            return api_utils.error_response(req, ex, headers, error)
+        except storage_errors.ExceptionBase as ex:
+            LOG.exception(ex)
+            headers = {'status': 503}
+            error = _('Cannot retrieve subscription %s.') % subscription_id
+            return api_utils.error_response(req, ex, headers, error)
+        else:
+            body = resp_dict
+            headers = {'status': 200}
+            return response.Response(req, body, headers)
