@@ -13,37 +13,31 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from oslo_config import cfg
+import socket
+
 from oslo_log import log
+from osprofiler import opts as profiler_opts
+from osprofiler import profiler
 from stevedore import driver
 
 from zaqar.api import handler
+from zaqar.common import cache as oslo_cache
 from zaqar.common import configs
+from zaqar.common import consts
 from zaqar.common import decorators
 from zaqar.common import errors
-from zaqar.openstack.common.cache import cache as oslo_cache
+from zaqar import profile
 from zaqar.storage import pipeline
 from zaqar.storage import pooling
 from zaqar.storage import utils as storage_utils
 from zaqar.transport import base
 from zaqar.transport import validation
 
+from zaqar.i18n import _LE
+
 LOG = log.getLogger(__name__)
 
 
-_CLI_OPTIONS = (
-    configs._ADMIN_MODE_OPT,
-    cfg.BoolOpt('daemon', default=False,
-                help='Run Zaqar server in the background.'),
-)
-
-# NOTE (Obulpathi): Register daemon command line option for
-# zaqar-server
-CONF = cfg.CONF
-CONF.register_cli_opts(_CLI_OPTIONS)
-log.register_options(CONF)
-
-
 class Bootstrap(object):
     """Defines the Zaqar bootstrapper.
 
@@ -56,11 +50,14 @@ class Bootstrap(object):
 
         for group, opts in configs._config_options():
             self.conf.register_opts(opts, group=group)
+        profiler_opts.set_defaults(self.conf)
+
+        # TODO(wangxiyuan): Now the OSprofiler feature in Zaqar only support
+        # wsgi. Websockt part will be added in the future.
+        profile.setup(self.conf, 'Zaqar-server', socket.gethostname())
 
         self.driver_conf = self.conf[configs._DRIVER_GROUP]
 
-        log.setup(conf, 'zaqar')
-
     @decorators.lazy_property(write=False)
     def api(self):
         LOG.debug(u'Loading API handler')
@@ -75,6 +72,9 @@ class Bootstrap(object):
             LOG.debug(u'Storage pooling enabled')
             storage_driver = pooling.DataDriver(self.conf, self.cache,
                                                 self.control)
+            if self.conf.profiler.enabled:
+                storage_driver = profiler.trace_cls("pooling_data_"
+                                                    "driver")(storage_driver)
         else:
             storage_driver = storage_utils.load_storage_driver(
                 self.conf, self.cache, control_driver=self.control)
@@ -93,9 +93,8 @@ class Bootstrap(object):
     def cache(self):
         LOG.debug(u'Loading proxy cache driver')
         try:
-            oslo_cache.register_oslo_configs(self.conf)
-            mgr = oslo_cache.get_cache(self.conf.cache_url)
-            return mgr
+            oslo_cache.register_config(self.conf)
+            return oslo_cache.get_cache(self.conf)
         except RuntimeError as exc:
             LOG.exception(exc)
             raise errors.InvalidDriver(exc)
@@ -105,8 +104,7 @@ class Bootstrap(object):
         transport_name = self.driver_conf.transport
         LOG.debug(u'Loading transport driver: %s', transport_name)
 
-        # FIXME(vkmc): Find a better way to init args
-        if transport_name == 'websocket':
+        if transport_name == consts.TRANSPORT_WEBSOCKET:
             args = [self.conf, self.api, self.cache]
         else:
             args = [
@@ -124,8 +122,8 @@ class Bootstrap(object):
             return mgr.driver
         except RuntimeError as exc:
             LOG.exception(exc)
-            LOG.error(u'Failed to load transport driver zaqar.transport.'
-                      u'%(driver)s with args %(args)s',
+            LOG.error(_LE(u'Failed to load transport driver zaqar.transport.'
+                          u'%(driver)s with args %(args)s'),
                       {'driver': transport_name, 'args': args})
             raise errors.InvalidDriver(exc)
 
