@@ -18,6 +18,7 @@ import itertools
 
 from oslo_config import cfg
 from oslo_log import log
+from osprofiler import profiler
 
 from zaqar.common import decorators
 from zaqar.common import errors as cerrors
@@ -74,7 +75,11 @@ class DataDriver(storage.DataDriverBase):
 
     def __init__(self, conf, cache, control, control_driver=None):
         super(DataDriver, self).__init__(conf, cache, control_driver)
-        self._pool_catalog = Catalog(conf, cache, control)
+        catalog = Catalog(conf, cache, control)
+        if self.conf.profiler.enabled:
+            catalog = profiler.trace_cls("pooling_catalogue_"
+                                         "controller")(catalog)
+        self._pool_catalog = catalog
 
     @property
     def capabilities(self):
@@ -121,19 +126,36 @@ class DataDriver(storage.DataDriverBase):
 
     @decorators.lazy_property(write=False)
     def queue_controller(self):
-        return QueueController(self._pool_catalog)
+        controller = QueueController(self._pool_catalog)
+        if self.conf.profiler.enabled:
+            return profiler.trace_cls("pooling_queue_controller")(controller)
+        else:
+            return controller
 
     @decorators.lazy_property(write=False)
     def message_controller(self):
-        return MessageController(self._pool_catalog)
+        controller = MessageController(self._pool_catalog)
+        if self.conf.profiler.enabled:
+            return profiler.trace_cls("pooling_message_controller")(controller)
+        else:
+            return controller
 
     @decorators.lazy_property(write=False)
     def claim_controller(self):
-        return ClaimController(self._pool_catalog)
+        controller = ClaimController(self._pool_catalog)
+        if self.conf.profiler.enabled:
+            return profiler.trace_cls("pooling_claim_controller")(controller)
+        else:
+            return controller
 
     @decorators.lazy_property(write=False)
     def subscription_controller(self):
-        return SubscriptionController(self._pool_catalog)
+        controller = SubscriptionController(self._pool_catalog)
+        if self.conf.profiler.enabled:
+            return (profiler.trace_cls("pooling_subscription_controller")
+                    (controller))
+        else:
+            return controller
 
 
 class QueueController(storage.Queue):
@@ -152,15 +174,14 @@ class QueueController(storage.Queue):
               limit=storage.DEFAULT_QUEUES_PER_PAGE, detailed=False):
 
         def all_pages():
-            cursor = self._pool_catalog._pools_ctrl.list(limit=0)
-            pools_list = list(next(cursor))
-            anypool = pools_list and pools_list[0]
-            yield next(self._pool_catalog.get_driver(anypool['name'])
-                       .queue_controller.list(
-                           project=project,
-                           marker=marker,
-                           limit=limit,
-                           detailed=detailed))
+            pool = self._pool_catalog.get_default_pool()
+            if pool is None:
+                raise errors.NoPoolFound()
+            yield next(pool.queue_controller.list(
+                project=project,
+                marker=marker,
+                limit=limit,
+                detailed=detailed))
 
         # make a heap compared with 'name'
         ls = heapq.merge(*[
@@ -186,7 +207,10 @@ class QueueController(storage.Queue):
             return {}
 
     def _create(self, name, metadata=None, project=None):
-        flavor = metadata and metadata.get('_flavor', None)
+        flavor = None
+        if isinstance(metadata, dict):
+            flavor = metadata.get('_flavor', None)
+
         self._pool_catalog.register(name, project=project, flavor=flavor)
 
         # NOTE(cpp-cabrera): This should always succeed since we just
@@ -407,6 +431,17 @@ class SubscriptionController(storage.Subscription):
             return control.exists(queue, subscription_id,
                                   project=project)
 
+    def confirm(self, queue, subscription_id, project=None, confirmed=None):
+        control = self._get_controller(queue, project)
+        if control:
+            return control.confirm(queue, subscription_id,
+                                   project=project, confirmed=confirmed)
+
+    def get_with_subscriber(self, queue, subscriber, project=None):
+        control = self._get_controller(queue, project)
+        if control:
+            return control.get_with_subscriber(queue, subscriber, project)
+
 
 class Catalog(object):
     """Represents the mapping between queues and pool drivers."""
@@ -487,15 +522,16 @@ class Catalog(object):
 
             if flavor is not None:
                 flavor = self._flavor_ctrl.get(flavor, project=project)
-                pools = self._pools_ctrl.get_group(group=flavor['pool'],
-                                                   detailed=True)
+                pools = self._pools_ctrl.get_pools_by_group(
+                    group=flavor['pool_group'],
+                    detailed=True)
                 pool = select.weighted(pools)
                 pool = pool and pool['name'] or None
             else:
                 # NOTE(flaper87): Get pools assigned to the default
                 # group `None`. We should consider adding a `default_group`
                 # option in the future.
-                pools = self._pools_ctrl.get_group(detailed=True)
+                pools = self._pools_ctrl.get_pools_by_group(detailed=True)
                 pool = select.weighted(pools)
                 pool = pool and pool['name'] or None
 
@@ -584,27 +620,14 @@ class Catalog(object):
         target = self.lookup(queue, project)
         return target and target.subscription_controller
 
-    def lookup(self, queue, project=None):
-        """Lookup a pool driver for the given queue and project.
-
-        :param queue: Name of the queue for which to find a pool
-        :param project: Project to which the queue belongs, or
-            None to specify the "global" or "generic" project.
-
-        :returns: A storage driver instance for the appropriate pool. If
-            the driver does not exist yet, it is created and cached. If the
-            queue is not mapped, returns None.
-        :rtype: Maybe DataDriver
-        """
-
-        try:
-            pool_id = self._pool_id(queue, project)
-        except errors.QueueNotMapped as ex:
-            LOG.debug(ex)
-
-            if not self._catalog_conf.enable_virtual_pool:
-                return None
+    def get_default_pool(self, use_listing=True):
+        if use_listing:
+            cursor = self._pools_ctrl.list(limit=0)
+            pools_list = list(next(cursor))
+            if pools_list:
+                return self.get_driver(pools_list[0]['name'])
 
+        if self._catalog_conf.enable_virtual_pool:
             conf_section = ('drivers:message_store:%s' %
                             self._conf.drivers.message_store)
 
@@ -638,6 +661,26 @@ class Catalog(object):
             # store.
             return self.get_driver(None, pool_conf)
 
+    def lookup(self, queue, project=None):
+        """Lookup a pool driver for the given queue and project.
+
+        :param queue: Name of the queue for which to find a pool
+        :param project: Project to which the queue belongs, or
+            None to specify the "global" or "generic" project.
+
+        :returns: A storage driver instance for the appropriate pool. If
+            the driver does not exist yet, it is created and cached. If the
+            queue is not mapped, returns None.
+        :rtype: Maybe DataDriver
+        """
+
+        try:
+            pool_id = self._pool_id(queue, project)
+        except errors.QueueNotMapped as ex:
+            LOG.debug(ex)
+
+            return self.get_default_pool(use_listing=False)
+
         return self.get_driver(pool_id)
 
     def get_driver(self, pool_id, pool_conf=None):
