@@ -19,7 +19,6 @@ import msgpack
 from oslo_utils import timeutils
 import redis
 
-from zaqar.common import decorators
 from zaqar.common import utils as common_utils
 from zaqar.storage import base
 from zaqar.storage import errors
@@ -31,7 +30,6 @@ SubscriptionEnvelope = models.SubscriptionEnvelope
 
 SUBSET_INDEX_KEY = 'subset_index'
 SUBSCRIPTION_IDS_SUFFIX = 'subscriptions'
-SUBSCRIBERS_SUFFIX = 'subscribers'
 
 
 class SubscriptionController(base.Subscription):
@@ -54,10 +52,6 @@ class SubscriptionController(base.Subscription):
                                       use_bin_type=True).pack
         self._unpacker = functools.partial(msgpack.unpackb, encoding='utf-8')
 
-    @decorators.lazy_property(write=False)
-    def _queue_ctrl(self):
-        return self.driver.queue_controller
-
     @utils.raises_conn_error
     @utils.retries_on_connection_error
     def list(self, queue, project=None, marker=None, limit=10):
@@ -66,19 +60,28 @@ class SubscriptionController(base.Subscription):
                                                       project,
                                                       SUBSCRIPTION_IDS_SUFFIX)
         rank = client.zrank(subset_key, marker)
-        start = rank + 1 if rank else 0
+        start = rank + 1 if rank is not None else 0
 
         cursor = (q for q in client.zrange(subset_key, start,
                                            start + limit - 1))
         marker_next = {}
 
         def denormalizer(record, sid):
+            now = timeutils.utcnow_ts()
+            ttl = int(record[2])
+            expires = int(record[3])
+            created = expires - ttl
+            is_confirmed = True
+            if len(record) == 6:
+                is_confirmed = record[5] == str(True)
             ret = {
                 'id': sid,
                 'source': record[0],
                 'subscriber': record[1],
-                'ttl': record[2],
-                'options': self._unpacker(record[3]),
+                'ttl': ttl,
+                'age': now - created,
+                'options': self._unpacker(record[4]),
+                'confirmed': is_confirmed,
             }
             marker_next['next'] = sid
 
@@ -90,13 +93,12 @@ class SubscriptionController(base.Subscription):
     @utils.raises_conn_error
     @utils.retries_on_connection_error
     def get(self, queue, subscription_id, project=None):
-
-        subscription = SubscriptionEnvelope.from_redis(subscription_id,
-                                                       self._client)
-        now = timeutils.utcnow_ts()
-
-        if subscription and not utils.subscription_expired_filter(subscription,
-                                                                  now):
+        subscription = None
+        if self.exists(queue, subscription_id, project):
+            subscription = SubscriptionEnvelope.from_redis(subscription_id,
+                                                           self._client)
+        if subscription:
+            now = timeutils.utcnow_ts()
             return subscription.to_basic(now)
         else:
             raise errors.SubscriptionDoesNotExist(subscription_id)
@@ -104,10 +106,6 @@ class SubscriptionController(base.Subscription):
     @utils.raises_conn_error
     @utils.retries_on_connection_error
     def create(self, queue, subscriber, ttl, options, project=None):
-        member_key = utils.scope_subscribers_set(queue, project,
-                                                 SUBSCRIBERS_SUFFIX)
-        if self._client.sismember(member_key, subscriber):
-            return None
         subscription_id = str(uuid.uuid4())
         subset_key = utils.scope_subscription_ids_set(queue,
                                                       project,
@@ -115,8 +113,8 @@ class SubscriptionController(base.Subscription):
 
         source = queue
         now = timeutils.utcnow_ts()
-        ttl = int(ttl)
         expires = now + ttl
+        confirmed = False
 
         subscription = {'id': subscription_id,
                         's': source,
@@ -124,21 +122,56 @@ class SubscriptionController(base.Subscription):
                         't': ttl,
                         'e': expires,
                         'o': self._packer(options),
-                        'p': project}
+                        'p': project,
+                        'c': confirmed}
 
-        if not self._queue_ctrl.exists(queue, project):
-            raise errors.QueueDoesNotExist(queue, project)
         try:
             # Pipeline ensures atomic inserts.
             with self._client.pipeline() as pipe:
-                pipe.sadd(member_key, subscriber)
-                pipe.zadd(subset_key, 1,
-                          subscription_id).hmset(subscription_id, subscription)
-                pipe.execute()
+                if not self._is_duplicated_subscriber(subscriber,
+                                                      queue,
+                                                      project):
+                    pipe.zadd(subset_key, 1,
+                              subscription_id).hmset(subscription_id,
+                                                     subscription)
+                    pipe.expire(subscription_id, ttl)
+                    pipe.execute()
+                else:
+                    return None
             return subscription_id
         except redis.exceptions.ResponseError:
             return None
 
+    def _is_duplicated_subscriber(self, subscriber, queue, project):
+        """Check if the subscriber is existing or not.
+
+        Given the limitation of Redis' expires(), it's hard to auto expire
+        subscriber from the set and subscription id from the sorted set, so
+        this method is used to do a ugly duplication check when adding a new
+        subscription so that we don't need the set for subscriber. And as a
+        side effect, this method will remove the unreachable subscription's id
+        from the sorted set.
+        """
+        subset_key = utils.scope_subscription_ids_set(queue,
+                                                      project,
+                                                      SUBSCRIPTION_IDS_SUFFIX)
+        try:
+            sub_ids = (q for q in self._client.zrange(subset_key, 0, -1))
+            for s_id in sub_ids:
+                subscription = self._client.hmget(s_id,
+                                                  ['s', 'u', 't', 'o', 'c'])
+                if subscription == [None, None, None, None, None]:
+                    # NOTE(flwang): Under this check, that means the
+                    # subscription has been expired. So redis can't get
+                    # the subscription but the id is still there. So let's
+                    # delete the id for clean up.
+                    self._client.zrem(subset_key, s_id)
+                if subscription[1] == subscriber:
+                    return True
+            return False
+        except redis.exceptions.ResponseError:
+            return True
+
     @utils.raises_conn_error
     @utils.retries_on_connection_error
     def exists(self, queue, subscription_id, project=None):
@@ -158,9 +191,40 @@ class SubscriptionController(base.Subscription):
         assert fields, ('`subscriber`, `ttl`, '
                         'or `options` not found in kwargs')
 
+        # Let's get our subscription by ID. If it does not exist,
+        # SubscriptionDoesNotExist error will be raised internally.
+        subscription_to_update = self.get(queue, subscription_id,
+                                          project=project)
+
+        new_subscriber = fields.get('u', None)
+
+        # Let's do some checks to prevent subscription duplication.
+        if new_subscriber:
+            # Check if 'new_subscriber' is really new for our subscription.
+            if subscription_to_update['subscriber'] != new_subscriber:
+                # It's new. We should raise error if this subscriber already
+                # exists for the queue and project.
+                if self._is_duplicated_subscriber(new_subscriber, queue,
+                                                  project):
+                    raise errors.SubscriptionAlreadyExists()
+
+        # NOTE(Eva-i): if there are new options, we need to pack them before
+        # sending to the database.
+        new_options = fields.get('o', None)
+        if new_options is not None:
+            fields['o'] = self._packer(new_options)
+
+        new_ttl = fields.get('t', None)
+        if new_ttl is not None:
+            now = timeutils.utcnow_ts()
+            expires = now + new_ttl
+            fields['e'] = expires
+
         # Pipeline ensures atomic inserts.
         with self._client.pipeline() as pipe:
             pipe.hmset(subscription_id, fields)
+            if new_ttl is not None:
+                pipe.expire(subscription_id, new_ttl)
             pipe.execute()
 
     @utils.raises_conn_error
@@ -168,12 +232,38 @@ class SubscriptionController(base.Subscription):
     def delete(self, queue, subscription_id, project=None):
         subset_key = utils.scope_subscription_ids_set(queue, project,
                                                       SUBSCRIPTION_IDS_SUFFIX)
-        member_key = utils.scope_subscribers_set(queue, project,
-                                                 SUBSCRIBERS_SUFFIX)
-        subscriber = self._client.hget(subscription_id, 'u')
-        # NOTE(prashanthr_): Pipelining is used to mitigate race conditions
+
+        if self._client.zrank(subset_key, subscription_id) is not None:
+            # NOTE(prashanthr_): Pipelining is used to mitigate race conditions
+            with self._client.pipeline() as pipe:
+                pipe.zrem(subset_key, subscription_id)
+                pipe.delete(subscription_id)
+                pipe.execute()
+
+    @utils.raises_conn_error
+    @utils.retries_on_connection_error
+    def get_with_subscriber(self, queue, subscriber, project=None):
+        subset_key = utils.scope_subscription_ids_set(queue,
+                                                      project,
+                                                      SUBSCRIPTION_IDS_SUFFIX)
+        sub_ids = (q for q in self._client.zrange(subset_key, 0, -1))
+        for s_id in sub_ids:
+            subscription = self._client.hmget(s_id,
+                                              ['s', 'u', 't', 'o', 'c'])
+            if subscription[1] == subscriber:
+                subscription = SubscriptionEnvelope.from_redis(s_id,
+                                                               self._client)
+                now = timeutils.utcnow_ts()
+                return subscription.to_basic(now)
+
+    @utils.raises_conn_error
+    @utils.retries_on_connection_error
+    def confirm(self, queue, subscription_id, project=None, confirmed=True):
+        # Let's get our subscription by ID. If it does not exist,
+        # SubscriptionDoesNotExist error will be raised internally.
+        self.get(queue, subscription_id, project=project)
+
+        fields = {'c': confirmed}
         with self._client.pipeline() as pipe:
-            pipe.srem(member_key, subscriber)
-            pipe.zrem(subset_key, subscription_id)
-            pipe.delete(subscription_id)
+            pipe.hmset(subscription_id, fields)
             pipe.execute()
