@@ -18,7 +18,7 @@ import os
 import fixtures
 from oslo_config import cfg
 from oslo_log import log
-import six
+from osprofiler import opts
 import testtools
 
 from zaqar.common import configs
@@ -57,6 +57,15 @@ class TestBase(testtools.TestCase):
         self.conf.register_opts(configs._GENERAL_OPTIONS)
         self.conf.register_opts(configs._DRIVER_OPTIONS,
                                 group=configs._DRIVER_GROUP)
+        self.conf.register_opts(configs._NOTIFICATION_OPTIONS,
+                                group=configs._NOTIFICATION_GROUP)
+        self.conf.register_opts(configs._NOTIFICATION_OPTIONS,
+                                group=configs._NOTIFICATION_GROUP)
+        self.conf.register_opts(configs._SIGNED_URL_OPTIONS,
+                                group=configs._SIGNED_URL_GROUP)
+        opts.set_defaults(self.conf)
+        self.conf.register_opts(configs._PROFILER_OPTIONS,
+                                group=configs._PROFILER_GROUP)
 
         self.mongodb_url = os.environ.get('ZAQAR_TEST_MONGODB_URL',
                                           'mongodb://127.0.0.1:27017')
@@ -100,8 +109,8 @@ class TestBase(testtools.TestCase):
         All overrides are automatically cleared at the end of the current
         test by the tearDown() method.
         """
-        for k, v in six.iteritems(kw):
-            self.conf.set_override(k, v, group)
+        for k, v in kw.items():
+            self.conf.set_override(k, v, group, enforce_type=True)
 
     def _my_dir(self):
         return os.path.abspath(os.path.dirname(__file__))
