@@ -19,6 +19,7 @@ import ddt
 import mock
 from oslo_utils import timeutils
 
+from zaqar.common import consts
 from zaqar.tests.unit.transport.websocket import base
 from zaqar.tests.unit.transport.websocket import utils as test_utils
 
@@ -39,16 +40,16 @@ class ClaimsBaseTest(base.V1_1Base):
             'X-Project-ID': self.project_id
         }
 
-        action = "queue_create"
+        action = consts.QUEUE_CREATE
         body = {"queue_name": "skittle"}
         req = test_utils.create_request(action, body, self.headers)
 
         with mock.patch.object(self.protocol, 'sendMessage') as msg_mock:
             self.protocol.onMessage(req, False)
             resp = json.loads(msg_mock.call_args[0][0])
-            self.assertEqual(resp['headers']['status'], 201)
+            self.assertEqual(201, resp['headers']['status'])
 
-        action = "message_post"
+        action = consts.MESSAGE_POST
         body = {"queue_name": "skittle",
                 "messages": [
                     {'body': 239, 'ttl': 300},
@@ -70,11 +71,11 @@ class ClaimsBaseTest(base.V1_1Base):
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 201)
+        self.assertEqual(201, resp['headers']['status'])
 
     def tearDown(self):
         super(ClaimsBaseTest, self).tearDown()
-        action = 'queue_delete'
+        action = consts.QUEUE_DELETE
         body = {'queue_name': 'skittle'}
 
         send_mock = mock.Mock()
@@ -84,11 +85,11 @@ class ClaimsBaseTest(base.V1_1Base):
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 204)
+        self.assertEqual(204, resp['headers']['status'])
 
     @ddt.data('[', '[]', '.', '"fail"')
     def test_bad_claim(self, doc):
-        action = "claim_create"
+        action = consts.CLAIM_CREATE
         body = doc
 
         send_mock = mock.Mock()
@@ -97,18 +98,18 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
-        action = "claim_update"
+        action = consts.CLAIM_UPDATE
         body = doc
 
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
     def test_exceeded_claim(self):
-        action = "claim_create"
+        action = consts.CLAIM_CREATE
         body = {"queue_name": "skittle",
                 "ttl": 100,
                 "grace": 60,
@@ -120,12 +121,12 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
     @ddt.data((-1, -1), (59, 60), (60, 59), (60, 43201), (43201, 60))
     def test_unacceptable_ttl_or_grace(self, ttl_grace):
         ttl, grace = ttl_grace
-        action = "claim_create"
+        action = consts.CLAIM_CREATE
         body = {"queue_name": "skittle",
                 "ttl": ttl,
                 "grace": grace}
@@ -136,13 +137,13 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
     @ddt.data(-1, 59, 43201)
     def test_unacceptable_new_ttl(self, ttl):
         claim = self._get_a_claim()
 
-        action = "claim_update"
+        action = consts.CLAIM_UPDATE
         body = {"queue_name": "skittle",
                 "claim_id": claim['body']['claim_id'],
                 "ttl": ttl}
@@ -153,10 +154,10 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
     def test_default_ttl_and_grace(self):
-        action = "claim_create"
+        action = consts.CLAIM_CREATE
         body = {"queue_name": "skittle"}
 
         send_mock = mock.Mock()
@@ -165,9 +166,9 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 201)
+        self.assertEqual(201, resp['headers']['status'])
 
-        action = "claim_get"
+        action = consts.CLAIM_GET
         body = {"queue_name": "skittle",
                 "claim_id": resp['body']['claim_id']}
 
@@ -175,12 +176,12 @@ class ClaimsBaseTest(base.V1_1Base):
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
 
-        self.assertEqual(resp['headers']['status'], 200)
+        self.assertEqual(200, resp['headers']['status'])
         self.assertEqual(self.defaults.claim_ttl, resp['body']['ttl'])
 
     def test_lifecycle(self):
         # First, claim some messages
-        action = "claim_create"
+        action = consts.CLAIM_CREATE
         body = {"queue_name": "skittle",
                 "ttl": 100,
                 "grace": 60}
@@ -191,7 +192,7 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 201)
+        self.assertEqual(201, resp['headers']['status'])
         claimed_messages = resp['body']['messages']
         claim_id = resp['body']['claim_id']
 
@@ -203,18 +204,18 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 204)
+        self.assertEqual(204, resp['headers']['status'])
 
         # Listing messages, by default, won't include claimed, will echo
-        action = "message_list"
+        action = consts.MESSAGE_LIST
         body = {"queue_name": "skittle",
                 "echo": True}
 
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 200)
-        self.assertEqual(resp['body']['messages'], [])
+        self.assertEqual(200, resp['headers']['status'])
+        self.assertEqual([], resp['body']['messages'])
 
         # Listing messages, by default, won't include claimed, won't echo
 
@@ -224,8 +225,8 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 200)
-        self.assertEqual(resp['body']['messages'], [])
+        self.assertEqual(200, resp['headers']['status'])
+        self.assertEqual([], resp['body']['messages'])
 
         # List messages, include_claimed, but don't echo
 
@@ -236,7 +237,7 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 200)
+        self.assertEqual(200, resp['headers']['status'])
         self.assertEqual(resp['body']['messages'], [])
 
         # List messages with a different client-id and echo=false.
@@ -253,7 +254,7 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 200)
+        self.assertEqual(200, resp['headers']['status'])
 
         # Include claimed messages this time, and echo
 
@@ -264,21 +265,21 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 200)
-        self.assertEqual(len(resp['body']['messages']), len(claimed_messages))
+        self.assertEqual(200, resp['headers']['status'])
+        self.assertEqual(len(claimed_messages), len(resp['body']['messages']))
 
         message_id_1 = resp['body']['messages'][0]['id']
         message_id_2 = resp['body']['messages'][1]['id']
 
         # Try to delete the message without submitting a claim_id
-        action = "message_delete"
+        action = consts.MESSAGE_DELETE
         body = {"queue_name": "skittle",
                 "message_id": message_id_1}
 
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 403)
+        self.assertEqual(403,  resp['headers']['status'])
 
         # Delete the message and its associated claim
         body = {"queue_name": "skittle",
@@ -288,7 +289,7 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 204)
+        self.assertEqual(204, resp['headers']['status'])
 
         # Try to get it from the wrong project
         headers = {
@@ -296,26 +297,26 @@ class ClaimsBaseTest(base.V1_1Base):
             'X-Project-ID': 'someproject'
         }
 
-        action = "message_get"
+        action = consts.MESSAGE_GET
         body = {"queue_name": "skittle",
                 "message_id": message_id_2}
         req = test_utils.create_request(action, body, headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 404)
+        self.assertEqual(404,  resp['headers']['status'])
 
         # Get the message
-        action = "message_get"
+        action = consts.MESSAGE_GET
         body = {"queue_name": "skittle",
                 "message_id": message_id_2}
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 200)
+        self.assertEqual(200, resp['headers']['status'])
 
         # Update the claim
         creation = timeutils.utcnow()
-        action = "claim_update"
+        action = consts.CLAIM_UPDATE
         body = {"queue_name": "skittle",
                 "ttl": 60,
                 "grace": 60,
@@ -323,35 +324,39 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 204)
+        self.assertEqual(204, resp['headers']['status'])
 
         # Get the claimed messages (again)
-        action = "claim_get"
+        action = consts.CLAIM_GET
         body = {"queue_name": "skittle",
                 "claim_id": claim_id}
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         query = timeutils.utcnow()
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 200)
-        self.assertEqual(resp['body']['ttl'], 60)
+        self.assertEqual(200, resp['headers']['status'])
+        self.assertEqual(60, resp['body']['ttl'])
 
         message_id_3 = resp['body']['messages'][0]['id']
 
         estimated_age = timeutils.delta_seconds(creation, query)
-        self.assertTrue(estimated_age > resp['body']['age'])
+        # The claim's age should be 0 at this moment. But in some unexpected
+        # case, such as slow test, the age maybe larger than 0. Just skip
+        # asserting if so.
+        if resp['body']['age'] == 0:
+            self.assertGreater(estimated_age, resp['body']['age'])
 
         # Delete the claim
-        action = "claim_delete"
+        action = consts.CLAIM_DELETE
         body = {"queue_name": "skittle",
                 "claim_id": claim_id}
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 204)
+        self.assertEqual(204, resp['headers']['status'])
 
         # Try to delete a message with an invalid claim ID
-        action = "message_delete"
+        action = consts.MESSAGE_DELETE
         body = {"queue_name": "skittle",
                 "message_id": message_id_3,
                 "claim_id": claim_id}
@@ -359,28 +364,28 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
         # Make sure it wasn't deleted!
-        action = "message_get"
+        action = consts.MESSAGE_GET
         body = {"queue_name": "skittle",
                 "message_id": message_id_2}
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 200)
+        self.assertEqual(200, resp['headers']['status'])
 
         # Try to get a claim that doesn't exist
-        action = "claim_get"
+        action = consts.CLAIM_GET
         body = {"queue_name": "skittle",
                 "claim_id": claim_id}
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 404)
+        self.assertEqual(404,  resp['headers']['status'])
 
         # Try to update a claim that doesn't exist
-        action = "claim_update"
+        action = consts.CLAIM_UPDATE
         body = {"queue_name": "skittle",
                 "ttl": 60,
                 "grace": 60,
@@ -388,10 +393,10 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 404)
+        self.assertEqual(404,  resp['headers']['status'])
 
     def test_post_claim_nonexistent_queue(self):
-        action = "claim_create"
+        action = consts.CLAIM_CREATE
         body = {"queue_name": "nonexistent",
                 "ttl": 100,
                 "grace": 60}
@@ -402,10 +407,10 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 204)
+        self.assertEqual(204, resp['headers']['status'])
 
     def test_get_claim_nonexistent_queue(self):
-        action = "claim_get"
+        action = consts.CLAIM_GET
         body = {"queue_name": "nonexistent",
                 "claim_id": "aaabbbba"}
 
@@ -415,10 +420,10 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 404)
+        self.assertEqual(404,  resp['headers']['status'])
 
     def _get_a_claim(self):
-        action = "claim_create"
+        action = consts.CLAIM_CREATE
         body = {"queue_name": "skittle",
                 "ttl": 100,
                 "grace": 60}
@@ -429,6 +434,6 @@ class ClaimsBaseTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 201)
+        self.assertEqual(201, resp['headers']['status'])
 
         return resp
