@@ -23,6 +23,7 @@ from oslo_utils import timeutils
 import six
 from testtools import matchers
 
+from zaqar.common import consts
 from zaqar.tests.unit.transport.websocket import base
 from zaqar.tests.unit.transport.websocket import utils as test_utils
 from zaqar.transport import validation
@@ -45,52 +46,53 @@ class MessagesBaseTest(base.V2Base):
             'X-Project-ID': self.project_id
         }
 
-        action = "queue_create"
         body = {"queue_name": "kitkat"}
-        req = test_utils.create_request(action, body, self.headers)
+        req = test_utils.create_request(consts.QUEUE_CREATE,
+                                        body, self.headers)
 
         with mock.patch.object(self.protocol, 'sendMessage') as msg_mock:
             self.protocol.onMessage(req, False)
             resp = json.loads(msg_mock.call_args[0][0])
-            self.assertEqual(resp['headers']['status'], 201)
+            self.assertEqual(201, resp['headers']['status'])
 
     def tearDown(self):
         super(MessagesBaseTest, self).tearDown()
-        action = "queue_delete"
         body = {"queue_name": "kitkat"}
 
         send_mock = mock.Mock()
         self.protocol.sendMessage = send_mock
 
-        req = test_utils.create_request(action, body, self.headers)
+        req = test_utils.create_request(consts.QUEUE_DELETE,
+                                        body, self.headers)
 
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 204)
+        self.assertEqual(204, resp['headers']['status'])
 
-    def _test_post(self, sample_messages):
-        action = "message_post"
+    def _test_post(self, sample_messages, in_binary=False):
         body = {"queue_name": "kitkat",
                 "messages": sample_messages}
 
         send_mock = mock.Mock()
         self.protocol.sendMessage = send_mock
 
-        req = test_utils.create_request(action, body, self.headers)
+        dumps, loads, create_req = test_utils.get_pack_tools(binary=in_binary)
 
-        self.protocol.onMessage(req, False)
+        req = create_req(consts.MESSAGE_POST, body, self.headers)
 
-        resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 201)
+        self.protocol.onMessage(req, in_binary)
+
+        resp = loads(send_mock.call_args[0][0])
+        self.assertEqual(201, resp['headers']['status'])
         self.msg_ids = resp['body']['message_ids']
-        self.assertEqual(len(self.msg_ids), len(sample_messages))
+        self.assertEqual(len(sample_messages), len(self.msg_ids))
 
         lookup = dict([(m['ttl'], m['body']) for m in sample_messages])
 
         # Test GET on the message resource directly
         # NOTE(cpp-cabrera): force the passing of time to age a message
-        timeutils_utcnow = 'zaqar.openstack.common.timeutils.utcnow'
+        timeutils_utcnow = 'oslo_utils.timeutils.utcnow'
         now = timeutils.utcnow() + datetime.timedelta(seconds=10)
         with mock.patch(timeutils_utcnow) as mock_utcnow:
             mock_utcnow.return_value = now
@@ -98,29 +100,29 @@ class MessagesBaseTest(base.V2Base):
                 headers = self.headers.copy()
                 headers['X-Project-ID'] = '777777'
                 # Wrong project ID
-                action = "message_get"
+                action = consts.MESSAGE_GET
                 body = {"queue_name": "kitkat",
                         "message_id": msg_id}
 
-                req = test_utils.create_request(action, body, headers)
+                req = create_req(action, body, headers)
 
-                self.protocol.onMessage(req, False)
+                self.protocol.onMessage(req, in_binary)
 
-                resp = json.loads(send_mock.call_args[0][0])
-                self.assertEqual(resp['headers']['status'], 404)
+                resp = loads(send_mock.call_args[0][0])
+                self.assertEqual(404, resp['headers']['status'])
 
                 # Correct project ID
-                req = test_utils.create_request(action, body, self.headers)
+                req = create_req(action, body, self.headers)
 
-                self.protocol.onMessage(req, False)
+                self.protocol.onMessage(req, in_binary)
 
-                resp = json.loads(send_mock.call_args[0][0])
-                self.assertEqual(resp['headers']['status'], 200)
+                resp = loads(send_mock.call_args[0][0])
+                self.assertEqual(200, resp['headers']['status'])
 
                 # Check message properties
                 message = resp['body']['messages']
-                self.assertEqual(message['body'], lookup[message['ttl']])
-                self.assertEqual(message['id'], msg_id)
+                self.assertEqual(lookup[message['ttl']], message['body'])
+                self.assertEqual(msg_id, message['id'])
 
                 # no negative age
                 # NOTE(cpp-cabrera): testtools lacks
@@ -129,15 +131,15 @@ class MessagesBaseTest(base.V2Base):
                                 matchers.GreaterThan(-1))
 
         # Test bulk GET
-        action = "message_get_many"
+        action = consts.MESSAGE_GET_MANY
         body = {"queue_name": "kitkat",
                 "message_ids": self.msg_ids}
-        req = test_utils.create_request(action, body, self.headers)
+        req = create_req(action, body, self.headers)
 
-        self.protocol.onMessage(req, False)
+        self.protocol.onMessage(req, in_binary)
 
-        resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 200)
+        resp = loads(send_mock.call_args[0][0])
+        self.assertEqual(200, resp['headers']['status'])
         expected_ttls = set(m['ttl'] for m in sample_messages)
         actual_ttls = set(m['ttl'] for m in resp['body']['messages'])
         self.assertFalse(expected_ttls - actual_ttls)
@@ -151,7 +153,7 @@ class MessagesBaseTest(base.V2Base):
 
         # Bulk GET restriction
         get_msg_ids = msg_id * 21
-        action = "message_get_many"
+        action = consts.MESSAGE_GET_MANY
         body = {"queue_name": "kitkat",
                 "message_ids": get_msg_ids}
 
@@ -162,46 +164,48 @@ class MessagesBaseTest(base.V2Base):
 
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
         # Listing restriction
         body['limit'] = 21
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
         # Bulk deletion restriction
         del_msg_ids = msg_id * 22
-        action = "message_get_many"
+        action = consts.MESSAGE_GET_MANY
         body = {"queue_name": "kitkat",
                 "message_ids": del_msg_ids}
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
-    def test_post_single(self):
+    @ddt.data(True, False)
+    def test_post_single(self, in_binary):
         sample_messages = [
             {'body': {'key': 'value'}, 'ttl': 200},
         ]
 
-        self._test_post(sample_messages)
+        self._test_post(sample_messages, in_binary=in_binary)
 
-    def test_post_multiple(self):
+    @ddt.data(True, False)
+    def test_post_multiple(self, in_binary):
         sample_messages = [
             {'body': 239, 'ttl': 100},
             {'body': {'key': 'value'}, 'ttl': 200},
             {'body': [1, 3], 'ttl': 300},
         ]
 
-        self._test_post(sample_messages)
+        self._test_post(sample_messages, in_binary=in_binary)
 
     def test_post_optional_ttl(self):
         messages = [{'body': 239},
                     {'body': {'key': 'value'}, 'ttl': 200}]
 
-        action = "message_post"
+        action = consts.MESSAGE_POST
         body = {"queue_name": "kitkat",
                 "messages": messages}
         req = test_utils.create_request(action, body, self.headers)
@@ -212,10 +216,10 @@ class MessagesBaseTest(base.V2Base):
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 201)
+        self.assertEqual(201, resp['headers']['status'])
         msg_id = resp['body']['message_ids'][0]
 
-        action = "message_get"
+        action = consts.MESSAGE_GET
         body = {"queue_name": "kitkat", "message_id": msg_id}
 
         req = test_utils.create_request(action, body, self.headers)
@@ -223,7 +227,7 @@ class MessagesBaseTest(base.V2Base):
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 200)
+        self.assertEqual(200, resp['headers']['status'])
         self.assertEqual(self.default_message_ttl,
                          resp['body']['messages']['ttl'])
 
@@ -234,7 +238,7 @@ class MessagesBaseTest(base.V2Base):
             queue_name = queue_name.encode('utf-8')
 
         resp = self._post_messages(queue_name)
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
     def test_post_with_long_queue_name(self):
         # NOTE(kgriffs): This test verifies that routes with
@@ -244,23 +248,23 @@ class MessagesBaseTest(base.V2Base):
         queue_name = 'v' * validation.QUEUE_NAME_MAX_LEN
 
         resp = self._post_messages(queue_name)
-        self.assertEqual(resp['headers']['status'], 201)
+        self.assertEqual(201, resp['headers']['status'])
 
         queue_name += 'v'
         resp = self._post_messages(queue_name)
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
     def test_post_to_missing_queue(self):
         queue_name = 'nonexistent'
         resp = self._post_messages(queue_name)
-        self.assertEqual(resp['headers']['status'], 201)
+        self.assertEqual(201, resp['headers']['status'])
 
     def test_post_invalid_ttl(self):
         sample_messages = [
             {'body': {'key': 'value'}, 'ttl': '200'},
         ]
 
-        action = "message_post"
+        action = consts.MESSAGE_POST
         body = {"queue_name": "kitkat",
                 "messages": sample_messages}
 
@@ -281,7 +285,7 @@ class MessagesBaseTest(base.V2Base):
             {'ttl': 200},
         ]
 
-        action = "message_post"
+        action = consts.MESSAGE_POST
         body = {"queue_name": "kitkat",
                 "messages": sample_messages}
 
@@ -297,7 +301,7 @@ class MessagesBaseTest(base.V2Base):
             'Bad request. Missing "body" field.', resp['body']['exception'])
 
     def test_get_from_missing_queue(self):
-        action = "message_list"
+        action = consts.MESSAGE_LIST
         body = {"queue_name": "anothernonexistent"}
         req = test_utils.create_request(action, body, self.headers)
 
@@ -307,12 +311,12 @@ class MessagesBaseTest(base.V2Base):
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 200)
-        self.assertEqual(resp['body']['messages'], [])
+        self.assertEqual(200, resp['headers']['status'])
+        self.assertEqual([], resp['body']['messages'])
 
     @ddt.data('', '0xdeadbeef', '550893e0-2b6e-11e3-835a-5cf9dd72369')
     def test_bad_client_id(self, text_id):
-        action = "message_post"
+        action = consts.MESSAGE_POST
         body = {
             "queue_name": "kinder",
             "messages": [{"ttl": 60,
@@ -331,9 +335,9 @@ class MessagesBaseTest(base.V2Base):
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
-        action = "message_get"
+        action = consts.MESSAGE_GET
         body = {
             "queue_name": "kinder",
             "limit": 3,
@@ -344,11 +348,11 @@ class MessagesBaseTest(base.V2Base):
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
     @ddt.data(None, '[', '[]', '{}', '.')
     def test_post_bad_message(self, document):
-        action = "message_post"
+        action = consts.MESSAGE_POST
         body = {
             "queue_name": "kinder",
             "messages": document
@@ -362,11 +366,11 @@ class MessagesBaseTest(base.V2Base):
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
     @ddt.data(-1, 59, 1209601)
     def test_unacceptable_ttl(self, ttl):
-        action = "message_post"
+        action = consts.MESSAGE_POST
         body = {"queue_name": "kinder",
                 "messages": [{"ttl": ttl, "body": ""}]}
 
@@ -378,12 +382,12 @@ class MessagesBaseTest(base.V2Base):
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
     def test_exceeded_message_posting(self):
         # Total (raw request) size
         document = [{'body': "some body", 'ttl': 100}] * 8000
-        action = "message_post"
+        action = consts.MESSAGE_POST
         body = {
             "queue_name": "kinder",
             "messages": document
@@ -397,12 +401,12 @@ class MessagesBaseTest(base.V2Base):
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
     @ddt.data('{"overflow": 9223372036854775808}',
               '{"underflow": -9223372036854775809}')
     def test_unsupported_json(self, document):
-        action = "message_post"
+        action = consts.MESSAGE_POST
         body = {
             "queue_name": "fizz",
             "messages": document
@@ -416,13 +420,13 @@ class MessagesBaseTest(base.V2Base):
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
     def test_delete(self):
         resp = self._post_messages("tofi")
         msg_id = resp['body']['message_ids'][0]
 
-        action = "message_get"
+        action = consts.MESSAGE_GET
         body = {"queue_name": "tofi",
                 "message_id": msg_id}
 
@@ -434,38 +438,38 @@ class MessagesBaseTest(base.V2Base):
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 200)
+        self.assertEqual(200, resp['headers']['status'])
 
         # Delete queue
-        action = "message_delete"
+        action = consts.MESSAGE_DELETE
         req = test_utils.create_request(action, body, self.headers)
 
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 204)
+        self.assertEqual(204, resp['headers']['status'])
 
         # Get non existent queue
-        action = "message_get"
+        action = consts.MESSAGE_GET
         req = test_utils.create_request(action, body, self.headers)
 
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 404)
+        self.assertEqual(404, resp['headers']['status'])
 
         # Safe to delete non-existing ones
-        action = "message_delete"
+        action = consts.MESSAGE_DELETE
         req = test_utils.create_request(action, body, self.headers)
 
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 204)
+        self.assertEqual(204, resp['headers']['status'])
 
     def test_bulk_delete(self):
         resp = self._post_messages("nerds", repeat=5)
         msg_ids = resp['body']['message_ids']
 
-        action = "message_delete_many"
+        action = consts.MESSAGE_DELETE_MANY
         body = {"queue_name": "nerds",
                 "message_ids": msg_ids}
 
@@ -477,45 +481,64 @@ class MessagesBaseTest(base.V2Base):
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 204)
+        self.assertEqual(204, resp['headers']['status'])
 
-        action = "message_get"
+        action = consts.MESSAGE_GET
         req = test_utils.create_request(action, body, self.headers)
 
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
         # Safe to delete non-existing ones
-        action = "message_delete_many"
+        action = consts.MESSAGE_DELETE_MANY
         req = test_utils.create_request(action, body, self.headers)
 
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 204)
+        self.assertEqual(204, resp['headers']['status'])
 
         # Even after the queue is gone
-        action = "queue_delete"
+        action = consts.QUEUE_DELETE
         body = {"queue_name": "nerds"}
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 204)
+        self.assertEqual(204, resp['headers']['status'])
 
-        action = "message_delete_many"
+        action = consts.MESSAGE_DELETE_MANY
         body = {"queue_name": "nerds",
                 "message_ids": msg_ids}
         req = test_utils.create_request(action, body, self.headers)
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 204)
+        self.assertEqual(204, resp['headers']['status'])
+
+    def test_pop_delete(self):
+        self._post_messages("kitkat", repeat=5)
+
+        action = consts.MESSAGE_DELETE_MANY
+        body = {"queue_name": "kitkat", "pop": 2}
+
+        send_mock = mock.Mock()
+        self.protocol.sendMessage = send_mock
+
+        req = test_utils.create_request(action, body, self.headers)
+
+        self.protocol.onMessage(req, False)
+
+        resp = json.loads(send_mock.call_args[0][0])
+        self.assertEqual(200, resp['headers']['status'])
+        self.assertEqual(2, len(resp['body']['messages']))
+        self.assertEqual(239, resp['body']['messages'][0]['body'])
+        self.assertEqual(239, resp['body']['messages'][1]['body'])
 
     def test_get_nonexistent_message_404s(self):
-        action = "message_get"
+        action = consts.MESSAGE_GET
         body = {"queue_name": "notthere",
                 "message_id": "a"}
 
@@ -526,10 +549,10 @@ class MessagesBaseTest(base.V2Base):
 
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 404)
+        self.assertEqual(404, resp['headers']['status'])
 
     def test_get_multiple_invalid_messages_404s(self):
-        action = "message_get_many"
+        action = consts.MESSAGE_GET_MANY
         body = {"queue_name": "notnotthere",
                 "message_ids": ["a", "b", "c"]}
 
@@ -540,10 +563,10 @@ class MessagesBaseTest(base.V2Base):
 
         self.protocol.onMessage(req, False)
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 200)
+        self.assertEqual(200, resp['headers']['status'])
 
     def test_delete_multiple_invalid_messages_204s(self):
-        action = "message_delete"
+        action = consts.MESSAGE_DELETE
         body = {"queue_name": "yetanothernotthere",
                 "message_ids": ["a", "b", "c"]}
 
@@ -555,12 +578,12 @@ class MessagesBaseTest(base.V2Base):
         self.protocol.onMessage(req, False)
 
         resp = json.loads(send_mock.call_args[0][0])
-        self.assertEqual(resp['headers']['status'], 400)
+        self.assertEqual(400, resp['headers']['status'])
 
     def _post_messages(self, queue_name, repeat=1):
         messages = [{'body': 239, 'ttl': 300}] * repeat
 
-        action = "message_post"
+        action = consts.MESSAGE_POST
         body = {"queue_name": queue_name,
                 "messages": messages}
 
