@@ -42,11 +42,14 @@ from oslo_log import log
 import six
 
 from zaqar.common.api.schemas import pools as schema
+from zaqar.common import decorators
 from zaqar.common import utils as common_utils
 from zaqar.i18n import _
 from zaqar.storage import errors
 from zaqar.storage import utils as storage_utils
+from zaqar.transport import acl
 from zaqar.transport import utils as transport_utils
+from zaqar.transport import validation
 from zaqar.transport.wsgi import errors as wsgi_errors
 from zaqar.transport.wsgi import utils as wsgi_utils
 
@@ -59,9 +62,12 @@ class Listing(object):
     :param pools_controller: means to interact with storage
     """
 
-    def __init__(self, pools_controller):
+    def __init__(self, pools_controller, validate):
         self._ctrl = pools_controller
+        self._validate = validate
 
+    @decorators.TransportLog("Pools collection")
+    @acl.enforce("pools:get_all")
     def on_get(self, request, response, project_id):
         """Returns a pool listing as objects embedded in an object:
 
@@ -87,6 +93,12 @@ class Listing(object):
         request.get_param_as_int('limit', store=store)
         request.get_param_as_bool('detailed', store=store)
 
+        try:
+            self._validate.pool_listing(**store)
+        except validation.ValidationFailed as ex:
+            LOG.debug(ex)
+            raise wsgi_errors.HTTPBadRequestAPI(six.text_type(ex))
+
         cursor = self._ctrl.list(**store)
         pools = list(next(cursor))
 
@@ -129,6 +141,8 @@ class Resource(object):
             'create': validator_type(schema.create)
         }
 
+    @decorators.TransportLog("Pools item")
+    @acl.enforce("pools:get")
     def on_get(self, request, response, project_id, pool):
         """Returns a JSON object for a single pool entry:
 
@@ -148,12 +162,13 @@ class Resource(object):
 
         except errors.PoolDoesNotExist as ex:
             LOG.debug(ex)
-            raise falcon.HTTPNotFound()
+            raise wsgi_errors.HTTPNotFound(six.text_type(ex))
 
         data['href'] = request.path
 
         response.body = transport_utils.to_json(data)
 
+    @acl.enforce("pools:create")
     def on_put(self, request, response, project_id, pool):
         """Registers a new pool. Expects the following input:
 
@@ -186,7 +201,12 @@ class Resource(object):
             LOG.exception(e)
             title = _(u'Unable to create pool')
             raise falcon.HTTPBadRequest(title, six.text_type(e))
+        except errors.PoolAlreadyExists as e:
+            LOG.exception(e)
+            raise wsgi_errors.HTTPConflict(six.text_type(e))
 
+    @decorators.TransportLog("Pools item")
+    @acl.enforce("pools:delete")
     def on_delete(self, request, response, project_id, pool):
         """Deregisters a pool.
 
@@ -207,6 +227,8 @@ class Resource(object):
 
         response.status = falcon.HTTP_204
 
+    @decorators.TransportLog("Pools item")
+    @acl.enforce("pools:update")
     def on_patch(self, request, response, project_id, pool):
         """Allows one to update a pool's weight, uri, and/or options.
 
@@ -242,9 +264,13 @@ class Resource(object):
             )
         fields = common_utils.fields(data, EXPECT,
                                      pred=lambda v: v is not None)
-
+        resp_data = None
         try:
             self._ctrl.update(pool, **fields)
+            resp_data = self._ctrl.get(pool, False)
         except errors.PoolDoesNotExist as ex:
             LOG.exception(ex)
-            raise falcon.HTTPNotFound()
+            raise wsgi_errors.HTTPNotFound(six.text_type(ex))
+
+        resp_data['href'] = request.path
+        response.body = transport_utils.to_json(resp_data)
